@@ -16,6 +16,7 @@ use File::Copy;
 use File::Copy::Recursive qw(rmove_glob);
 use File::Path qw(make_path rmtree);
 use File::Find::Rule;
+use JSON;
 use Module::Starter;
 use PPI;
 use Term::ReadKey;
@@ -31,25 +32,36 @@ our @EXPORT_OK = qw(
     changes_date
     ci_badges
     ci_github
+    config
+    config_file
+    copyright_info
+    copyright_bump
     cpan_upload
     git_add
     git_commit
+    git_clone
     git_pull
     git_push
     git_ignore
     git_release
+    git_repo
+    git_status_differs
     git_tag
     init
     make_dist
     make_distclean
+    make_manifest
     make_test
     manifest_skip
+    manifest_t
     move_distribution_files
     remove_unwanted_files
     version_bump
+    version_incr
     version_info
 );
 our @EXPORT_PRIVATE = qw(
+    _dist_dir_re
     _validate_git
 );
 our %EXPORT_TAGS = (
@@ -57,9 +69,10 @@ our %EXPORT_TAGS = (
     private => _export_private(),
 );
 
-our $VERSION = '1.00';
+our $VERSION = '1.06_01';
 
 use constant {
+    CONFIG_FILE         => 'dist-mgr.json',
     GITHUB_CI_FILE      => 'github_ci_default.yml',
     GITHUB_CI_PATH      => '.github/workflows/',
     CHANGES_FILE        => 'Changes',
@@ -67,6 +80,8 @@ use constant {
     FSTYPE_IS_DIR       => 1,
     FSTYPE_IS_FILE      => 2,
     DEFAULT_DIR         => 'lib/',
+    DEFAULT_POD_DIR     => '.',
+    MAKE                => $^O =~ /win32/i ? 'gmake' : 'make',
 };
 
 # Public
@@ -100,13 +115,19 @@ sub changes {
 
     $file //= 'Changes';
 
-    # Overwrite the Changes file if the SHA1 checksum matches the original file
-    # created with module-starter
+    # Overwrite the Changes file if there aren't any dates in it
 
     my @contents;
 
-    if (! -e $file || _sha1sum($file) eq CHANGES_ORIG_SHA) {
-        @contents = _changes_file($module);
+    my $changes_date_count = 0;
+
+    if (-e $file) {
+        my ($contents, $tie) = _changes_tie($file);
+        $changes_date_count = grep /\d{4}-\d{2}-\d{2}/, $contents;
+        untie $tie;
+    }
+    if (! -e $file || ! $changes_date_count) {
+        my @contents = _changes_file($module);
         _changes_write_file($file, \@contents);
     }
 
@@ -142,6 +163,9 @@ sub changes_date {
     $y += 1900;
     $m += 1;
 
+    $m = "0$m" if length $m == 1;
+    $d = "0$d" if length $d == 1;
+
     for (0..$#$contents) {
         if ($contents->[$_] =~ /^(.*)\s+UNREL/) {
             $contents->[$_] = "$1    $y-$m-$d";
@@ -152,23 +176,50 @@ sub changes_date {
     untie $tie;
 }
 sub ci_badges {
-    if (scalar @_ != 3) {
-        croak("ci_badges() needs \$author, \$repo and \$fs_entry sent in");
+    if (scalar @_ < 2) {
+        croak("ci_badges() needs \$author and \$repo sent in");
     }
 
     my ($author, $repo, $fs_entry) = @_;
 
+    $fs_entry //= DEFAULT_DIR;
+
+    my $exit = 0;
+
     for (_module_find_files($fs_entry)) {
-        _module_insert_ci_badges($author, $repo, $_);
+        $exit = -1 if _module_insert_ci_badges($author, $repo, $_) == -1;
     }
 
-    return 0;
+    return $exit;
 }
 sub ci_github {
     my ($os) = @_;
 
     if (defined $os && ref $os ne 'ARRAY') {
-        croak("\$os parameter to github_ci() must be an array ref");
+        croak("\$os parameter to ci_github() must be an array ref");
+    }
+
+    # Add the CI file to MANIFEST.SKIP
+
+    if (-e 'MANIFEST.SKIP') {
+        open my $fh, '<', 'MANIFEST.SKIP'
+            or croak("Can't open MANIFEST.SKIP for reading");
+
+        my @makefile_skip_contents = <$fh>;
+
+        if (grep !m|\.github$|, @makefile_skip_contents) {
+            close $fh;
+            open my $wfh, '>>', 'MANIFEST.SKIP'
+                or croak("Can't open MANIFEST.SKIP for writing");
+
+            print $wfh '^\.github/';
+        }
+    }
+    else {
+        open my $wfh, '>>', 'MANIFEST.SKIP'
+            or croak("Can't open MANIFEST.SKIP for writing");
+
+        print $wfh '^\.github/';
     }
 
     my @contents = _ci_github_file($os);
@@ -176,9 +227,100 @@ sub ci_github {
 
     return @contents;
 }
+sub config {
+    my ($args, $file) = @_;
+
+    if (! defined $args) {
+        croak("config() requires \$args hash reference parameter");
+    }
+    elsif (ref $args ne 'HASH') {
+        croak("\$args parameter must be a hash reference.");
+    }
+
+    $file = config_file() if ! defined $file;
+    my $conf;
+
+    if (-e $file && -f $file) {
+        {
+            local $/;
+            open my $fh, '<', $file or croak "Can't open config file $file: $!";
+            my $json = <$fh>;
+            $conf = decode_json $json;
+
+            for (keys %{ $conf }) {
+                delete $conf->{$_} if $conf->{$_} eq '';
+            }
+        }
+    }
+    else {
+        # No config file present
+        _config_file_write($file, _config_file());
+
+        print "\nGenerated new configuration file: $file\n";
+    }
+
+    %{ $args } = (%{ $args }, %{ $conf }) if $conf;
+
+    return $args;
+}
+sub config_file {
+    my $file = $^O =~ /win32/i
+        ? "$ENV{USERPROFILE}/${\CONFIG_FILE}"
+        : "$ENV{HOME}/${\CONFIG_FILE}";
+
+    return $file;
+}
+sub copyright_bump {
+    my ($fs_entry) = @_;
+
+    $fs_entry //= DEFAULT_POD_DIR;
+    _validate_fs_entry($fs_entry);
+
+    my ($year) = (localtime)[5];
+    $year += 1900;
+
+    my @pod_files = _pod_find_files($fs_entry);
+    my %info;
+
+    for my $pod_file (@pod_files) {
+        my ($contents, $tie) = _pod_tie($pod_file);
+
+        for (0 .. $#$contents) {
+            if ($contents->[$_] =~ /^(Copyright\s+)\d{4}(\s+.*)/) {
+                $contents->[$_] = "$1$year$2";
+                $info{$pod_file} = $year;
+                last;
+            }
+        }
+        untie $tie;
+    }
+
+    return \%info;
+}
+sub copyright_info {
+    my ($fs_entry) = @_;
+
+    $fs_entry //= DEFAULT_POD_DIR;
+
+    _validate_fs_entry($fs_entry);
+
+    my @pod_files = _pod_find_files($fs_entry);
+
+    my %copyright_info;
+
+    for my $file (@pod_files) {
+        my $copyright = _pod_extract_file_copyright($file);
+        next if ! defined $copyright || $copyright !~ /^\d{4}$/;
+        $copyright_info{$file} = $copyright if defined $copyright;
+    }
+
+    return \%copyright_info;
+}
 sub cpan_upload {
     my ($dist_file_name, %args) = @_;
 
+    config(\%args);
+
     if (! defined $dist_file_name) {
         croak("cpan_upload() requires the name of a distribution file sent in");
     }
@@ -187,17 +329,28 @@ sub cpan_upload {
         croak("File name sent to cpan_upload() isn't a valid file");
     }
 
-    $args{user} = $ENV{CPAN_USERNAME} ||= 0 if ! exists $args{user};
-    $args{password} = $ENV{CPAN_PASSWORD} ||= 0 if ! exists $args{password};
+    $args{user}     //= $args{cpan_id};
+    $args{password} //= $args{cpan_pw};
+
+    $args{user} = $ENV{CPAN_USERNAME} if ! $args{user};
+    $args{password} = $ENV{CPAN_PASSWORD} if ! $args{password};
 
     if (! $args{user} || ! $args{password}) {
-        croak("cpan_upload() requires the CPAN_USERNAME and CPAN_PASSWORD env vars set");
+        croak("\ncpan_upload() requires --cpan_id and --cpan_pw");
+    }
+
+    if ($args{dry_run}) {
+        print "\nCPAN upload is in dry run mode... nothing will be uploaded\n";
     }
 
     CPAN::Uploader->upload_file(
         $dist_file_name,
         \%args
     );
+
+    print "\nSuccessfully uploaded $dist_file_name to the CPAN\n";
+
+    return %args;
 }
 sub git_add {
     _git_add();
@@ -216,6 +369,9 @@ sub git_ignore {
 sub git_commit {
     _git_commit(@_);
 }
+sub git_clone {
+    _git_clone(@_);
+}
 sub git_push {
     _git_push(@_);
 }
@@ -225,15 +381,23 @@ sub git_pull {
 sub git_release {
     _git_release(@_);
 }
+sub git_repo {
+    _git_repo();
+}
+sub git_status_differs {
+    _git_status_differs(@_);
+}
 sub git_tag {
     _git_tag(@_);
 }
 sub init {
     my (%args) = @_;
 
+    config(\%args);
+
     my $cwd = getcwd();
 
-    if ($cwd =~ /dist-mgr$/) {
+    if ($cwd =~ _dist_dir_re()) {
         croak "Can't run init() while in the '$cwd' directory";
     }
 
@@ -250,7 +414,15 @@ sub init {
         croak("init()'s 'modules' parameter must be an array reference");
     }
 
-    Module::Starter->create_distro(%args);
+    if ($args{verbose}) {
+        delete $args{verbose};
+        Module::Starter->create_distro(%args);
+    }
+    else {
+        capture_merged {
+            Module::Starter->create_distro(%args);
+        };
+    }
 
     my ($module) = (@{ $args{modules} })[0];
     my $module_file = $module;
@@ -280,6 +452,17 @@ sub manifest_skip {
 
     return @content;
 }
+sub manifest_t {
+    my ($dir) = @_;
+
+    $dir //= './t';
+
+    my @content = _manifest_t_file();
+
+    _manifest_t_write_file($dir, \@content);
+
+    return @content;
+}
 sub move_distribution_files {
     my ($module) = @_;
 
@@ -315,38 +498,75 @@ sub remove_unwanted_files {
     for (_unwanted_filesystem_entries()) {
         rmtree $_;
     }
-
+    make_manifest();
     return 0;
 }
 sub make_dist {
-    capture_merged {
-        `make dist`;
-    };
+    my ($verbose) = @_;
+
+    my $cmd = "${\MAKE} dist";
+    $verbose ? `$cmd` : capture_merged {`$cmd`};
 
     if ($? != 0) {
-        croak("Exit code $? returned... 'make dist' failed");
+        croak("Exit code $? returned... '${\MAKE} dist' failed");
     }
 
     return $?;
 }
 sub make_distclean {
-    capture_merged {
-        `make distclean`;
-    };
+    my ($verbose) = @_;
+
+    my $cmd = "${\MAKE} distclean";
+    $verbose ? print `$cmd` : capture_merged {`$cmd`};
 
     if ($? != 0) {
-        croak("Exit code $? returned... 'make dist' failed");
+        croak("Exit code $? returned... '${\MAKE} distclean' failed\n");
+    }
+
+    return $?;
+}
+sub make_manifest {
+    my ($verbose) = @_;
+
+    if ($verbose) {
+        if (-f 'MANIFEST') {
+            unlink 'MANIFEST' or die "make_manifest() Couldn't remove MANIFEST\n";
+        }
+        print `$^X Makefile.PL`;
+        print `${\MAKE} manifest`;
+        make_distclean($verbose);
+    }
+    else {
+        capture_merged {
+            if (-f 'MANIFEST') {
+                unlink 'MANIFEST' or die "make_manifest() Couldn't remove MANIFEST\n";
+            }
+            `$^X Makefile.PL`;
+            `${\MAKE} manifest`;
+            make_distclean($verbose);
+        };
+    }
+
+    if ($? != 0) {
+        croak("Exit code $? returned... '${\MAKE} manifest' failed\n");
     }
 
     return $?;
 }
 sub make_test {
+    my ($verbose) = @_;
+
+    if ($verbose) {
+        print `$^X Makefile.PL`;
+        print `${\MAKE} test`;
+    }
     capture_merged {
-        `$^X Makefile.PL && make test`;
+        `$^X Makefile.PL`;
+        `${\MAKE} test`;
     };
 
     if ($? != 0) {
-        croak("Exit code $? returned... 'make test' failed");
+        croak("Exit code $? returned... '${\MAKE} test' failed\n");
     }
 
     return $?;
@@ -362,6 +582,8 @@ sub version_bump {
         $dry_run = 1;
     }
 
+    $fs_entry //= DEFAULT_DIR;
+
     _validate_version($version);
     _validate_fs_entry($fs_entry);
 
@@ -421,9 +643,21 @@ sub version_bump {
     }
     return \%files;
 }
+sub version_incr {
+    my ($version) = @_;
+
+    croak("version_incr() needs a version number sent in") if ! defined $version;
+
+    my $incremented_version;
+
+    _validate_version($version);
+    return sprintf("%.2f", $version + '0.01');
+}
 sub version_info {
     my ($fs_entry) = @_;
 
+    $fs_entry //= DEFAULT_DIR;
+
     _validate_fs_entry($fs_entry);
 
     my @module_files = _module_find_files($fs_entry);
@@ -485,6 +719,25 @@ sub _ci_github_write_file {
     print $fh "$_\n" for @$contents;
 }
 
+# Configuration related
+
+sub _config_file_write {
+    my ($file, $contents) = @_;
+
+    if (ref $contents ne 'HASH') {
+        croak("_config_file_write() requires a hash ref of contents");
+    }
+
+    my $jobj = JSON->new;
+
+    my $json = $jobj->pretty->encode($contents);
+
+    open my $fh, '>', $file or croak "Can't open config $file for writing: $!";
+
+    print $fh $json;
+
+}
+
 # Distribution related
 
 sub _default_distribution_file_count {
@@ -558,6 +811,8 @@ sub _makefile_insert_bugtracker {
 
     my ($mf, $tie) = _makefile_tie($makefile);
 
+    return -1 if grep /bugtracker/, @$mf;
+
     if (grep ! /META_MERGE/, @$mf) {
         _makefile_insert_meta_merge($mf);
     }
@@ -583,6 +838,8 @@ sub _makefile_insert_repository {
 
     my ($mf, $tie) = _makefile_tie($makefile);
 
+    return -1 if grep /repository/, @$mf;
+
     if (grep ! /META_MERGE/, @$mf) {
         _makefile_insert_meta_merge($mf);
     }
@@ -598,7 +855,7 @@ sub _makefile_insert_repository {
     return 0;
 }
 
-# MANIFEST.SKIP related
+# MANIFEST related
 
 sub _manifest_skip_write_file {
     # Writes out the MANIFEST.SKIP file
@@ -613,6 +870,20 @@ sub _manifest_skip_write_file {
 
     return 0;
 }
+sub _manifest_t_write_file {
+    # Writes out the t/manifest.t test file
+
+    my ($dir, $content) = @_;
+
+    open my $fh, '>', "$dir/manifest.t"
+        or croak("Can't open t/manifest.t for writing: $!\n");
+
+    for (@$content) {
+        print $fh "$_\n"
+    }
+
+    return 0;
+}
 
 # Module related
 
@@ -706,6 +977,8 @@ sub _module_insert_ci_badges {
 
     my ($mf, $tie) = _module_tie($module_file);
 
+    return -1 if grep /badge\.svg/, @$mf;
+
     for (0..$#$mf) {
         if ($mf->[$_] =~ /^=head1 NAME/) {
             splice @$mf, $_+3, 0, _module_section_ci_badges($author, $repo);
@@ -756,18 +1029,67 @@ sub _module_write_template {
     print $wfh "$_\n" for @content;
 }
 
+# POD related
+
+sub _pod_extract_file_copyright {
+    # Extracts the copyright year from POD
+
+    my ($module_file) = @_;
+
+    my $copyright_line = _pod_extract_file_copyright_line($module_file);
+
+    if (defined $copyright_line) {
+        if ($copyright_line =~ /^Copyright\s+(\d{4})\s+\w+/) {
+            return $1;
+        }
+    }
+    else {
+        warn("$_: Can't find a Copyright definition\n");
+    }
+    return undef;
+}
+sub _pod_extract_file_copyright_line {
+    # Extracts the Copyright line from a module file
+
+    my ($pod_file) = @_;
+
+    open my $fh, '<', $pod_file or croak("Can't open POD file $pod_file: $!");
+
+    while (<$fh>) {
+        if (/^Copyright\s+\d{4}\s+\w+/) {
+            return $_;
+        }
+    }
+}
+sub _pod_find_files {
+    # Finds POD files
+
+    my ($fs_entry) = @_;
+
+    $fs_entry //= DEFAULT_POD_DIR;
+
+    return File::Find::Rule->file()
+        ->name('*.pod', '*.pm', '*.pl')
+        ->in($fs_entry);
+}
+sub _pod_tie {
+    # Ties a POD file to an array
+
+    my ($pod_file) = @_;
+    croak("_pod_tie() needs a POD file name sent in") if ! defined $pod_file;
+
+    my $tie = tie my @pf, 'Tie::File', $pod_file;
+    return (\@pf, $tie);
+}
+
 # Validation related
 
-sub _sha1sum {
-    my ($file) = @_;
+sub _dist_dir_re {
+    # Capture permutations of the distribution directory for various
+    # CPAN testers
+    # Use YAPE::Regex::Explain for details
 
-    croak("shasum needs file param") if ! defined $file;
-
-    my $sha1 = Digest::SHA->new;
-
-    $sha1->addfile($file, 'U');
-
-    return $sha1->hexdigest;
+    return qr/dist-mgr(?:-\d+\.\d+)?(?:-\w+)?$/i;
 }
 sub _validate_git {
     my $sep = $^O =~ /win32/i ? ';' : ':';
@@ -814,7 +1136,7 @@ Steve Bertrand, C<< <steveb at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2020 Steve Bertrand.
+Copyright 2021 Steve Bertrand.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
@@ -822,16 +1144,3 @@ copy of the full license at:
 
 L<http://www.perlfoundation.org/artistic_license_2_0>
 
-=head1 AUTHOR
-
-Steve Bertrand, C<< <steveb at cpan.org> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 2020 Steve Bertrand.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the the Artistic License (2.0). You may obtain a
-copy of the full license at:
-
-L<http://www.perlfoundation.org/artistic_license_2_0>
