@@ -3,6 +3,8 @@ package Dist::Mgr::FileData;
 use warnings;
 use strict;
 
+our $VERSION = '1.06_01';
+
 use Exporter qw(import);
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
@@ -10,6 +12,8 @@ our @EXPORT_OK = qw(
 
     _ci_github_file
 
+    _config_file
+
     _git_ignore_file
 
     _module_section_ci_badges
@@ -20,6 +24,7 @@ our @EXPORT_OK = qw(
     _makefile_section_repo
 
     _manifest_skip_file
+    _manifest_t_file
 
     _unwanted_filesystem_entries
 );
@@ -100,6 +105,13 @@ sub _ci_github_file {
     );
 }
 
+sub _config_file {
+    return {
+        cpan_id     => '',
+        cpan_pw     => '',
+    };
+}
+
 sub _git_ignore_file {
     return (
         q{Makefile},
@@ -263,6 +275,7 @@ sub _manifest_skip_file {
         q{pm_to_blib$},
         q{.git/},
         q{.debug$},
+        q{^\.github/},
         q{.gitignore$},
         q{^\w+.pl$},
         q{.ignore.txt$},
@@ -279,12 +292,31 @@ sub _manifest_skip_file {
         q{scrap\.pl},
     );
 }
+sub _manifest_t_file {
+    return (
+        q|use warnings;|,
+        q|use strict;|,
+        q||,
+        q|use Test::More;|,
+        q|use ExtUtils::Manifest;|,
+        q||,
+        q|if (! $ENV{RELEASE_TESTING}) {|,
+        q|    plan skip_all => "Author tests not required for installation";|,
+        q|}|,
+        q||,
+        q|is_deeply [ ExtUtils::Manifest::manicheck() ], [], 'missing';|,
+        q|is_deeply [ ExtUtils::Manifest::filecheck() ], [], 'extra';|,
+        q||,
+        q|done_testing;|,
+    );
+}
 
 sub _unwanted_filesystem_entries {
     return qw(
         xt/
         ignore.txt
         README
+        MANIFEST
     );
 }
 
@@ -333,6 +365,11 @@ internally.
         w => qq{windows-latest},
         m => qq{macos-latest},
 
+=head2 _config_file
+
+Returns a hash reference containing the data that makes up a default
+configuration file.
+
 =head2 _git_ignore_file()
 
 Returns an array of the contents of a populated C<.gitignore> file.
@@ -366,6 +403,11 @@ C<Makefile.PL> file.
 
 Returns an array of the file lines that make up a default C<MANIFEST.SKIP> file.
 
+=head2 _manifest_t_file()
+
+Returns an array of the file lines that make up our custom C<t/makefile.t> test
+file.
+
 =head2 _unwanted_filesystem_entries()
 
 Returns an array of files and directories we remove from the base, stock
@@ -389,7 +431,7 @@ Steve Bertrand, C<< <steveb at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2020 Steve Bertrand.
+Copyright 2020-2021 Steve Bertrand.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
@@ -397,16 +439,3 @@ copy of the full license at:
 
 L<http://www.perlfoundation.org/artistic_license_2_0>
 
-=head1 AUTHOR
-
-Steve Bertrand, C<< <steveb at cpan.org> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 2020 Steve Bertrand.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the the Artistic License (2.0). You may obtain a
-copy of the full license at:
-
-L<http://www.perlfoundation.org/artistic_license_2_0>
